--------------------------------------
-- Namespaces
--------------------------------------
local _, core = ...
core.AltVaultCovenant = {}

local AltVault = core.AltVault
local AltVaultCovenant = core.AltVaultCovenant

local NUM_POWER_ROWS = 12
local ANIMA_ID = 1813
local SOULS_ID = 1810

local SOULSHOT_NAME = {
    ["Necrolord"] = "Necrolords",
    ["Kyrian"] = "Kyrian",
    ["Venthyr"] = "Venthyr",
    ["NightFae"] = "Fey"
}

local SOULSHOT_MAPPING = {
    ["Niya"] = {
        ["left"] = 0.591309,
        ["right"] = 0.873535
    },
    ["Dreamweaver"] = {
        ["left"] = 0.000488281,
        ["right"] = 0.294922
    },
    ["Korayn"] = {
        ["left"] = 0.295898,
        ["right"] = 0.590332
    },
    ["Pelagos"] = {
        ["left"] = 0.579102,
        ["right"] = 0.861328
    },
    ["Kleia"] = {
        ["left"] = 0.000488281,
        ["right"] = 0.294922
    },
    ["Mikanikos"] = {
        ["left"] = 0.295898,
        ["right"] = 0.578125
    },
    ["Nadjia"] = {
        ["left"] = 0.000488281,
        ["right"] = 0.307617
    },
    ["Theotar"] = {
        ["left"] = 0.308594,
        ["right"] = 0.605469
    },
    ["Draven"] = {
        ["left"] = 0.606445,
        ["right"] = 0.900879
    },
    ["Marileth"] = {
        ["left"] = 0.588867,
        ["right"] = 0.868164
    },
    ["Emeni"] = {
        ["left"] = 0.000488281,
        ["right"] = 0.307617
    },
    ["Heirmir"] = {
        ["left"] = 0.308594,
        ["right"] = 0.587891
    },
}

local SOULSHOT_TOP = 0.000976562
local SOULSHOT_BOTTOM = 0.545898


function AltCovenantPage_OnShow()
    local characterData = AltVaultDB.characters[CharacterJournal.activeAlt]
    if characterData.covenant == nil or characterData.covenant.covenantID == nil then
        AltCovenantContainer:Hide()
        AltCovenantNotAvailable:Show()
    else
        AltCovenantContainer:Show()
        SetupCovenantTheme()
        SetupCovenantRenown()
        SetupCovenantTalents()
        SetupCovenantSanctum()
        AltCovenantNotAvailable:Hide()
    end
end

function SetupCovenantRenown()
    local characterData = AltVaultDB.characters[CharacterJournal.activeAlt]
    RenownLevel:SetText(characterData.covenant.renownLevel)
    RewardsLabel:Hide()
    NextReward1:Hide()
    NextReward2:Hide()
    local rewardsData = characterData.covenant.nextRewards
    if rewardsData and #rewardsData ~= 0 then
        RewardsLabel:SetText("Next Reward")
        RewardsLabel:Show()
        local reward1 = rewardsData[1]
        NextReward1:SetID(1)
        NextReward1.Icon:SetTexture(reward1.rewardIcon)
        NextReward1:Show()
        -- Sometimes there are 2
        if #rewardsData == 2 then
            local reward2 = rewardsData[2]
            NextReward2:SetID(2)
            NextReward2.Icon:SetTexture(reward2.rewardIcon)
            RewardsLabel:SetText("Next Rewards")
            NextReward2:Show()
        end
    end
end

function SetupCovenantTheme()
    local characterData = AltVaultDB.characters[CharacterJournal.activeAlt]
    local covenantData = C_Covenants.GetCovenantData(characterData.covenant.covenantID)
    local covenantName = covenantData.name
    local nameKey = covenantData.textureKit

    -- Covenant Name
    CovenantName:SetText(covenantName)

    RenownContainer:SetAtlas("covenantsanctum-renownlevel-border-"..nameKey)
    -- Covenant Sigil
    CovenantSigil:SetAtlas("ShadowlandsMissionsLandingPage-Background-"..nameKey)

    -- Sanctum Container background -Necros are weird
    if nameKey == 'Necrolord' then
        SanctumFrameBackground:SetAtlas("ui-frame-"..SOULSHOT_NAME[nameKey].."choice-scrollingbg")
    else
        SanctumFrameBackground:SetAtlas("ui-frame-"..nameKey.."choice-scrollingbg")
    end
    
    -- Sanctum Container Frame
    SanctumFrameLeft:SetAtlas("!"..nameKey.."-NineSlice-EdgeLeft")
    SanctumFrameRight:SetAtlas("!"..nameKey.."-NineSlice-EdgeRight")
    SanctumFrameBottom:SetAtlas("_"..nameKey.."-NineSlice-EdgeBottom")
    SanctumFrameBottomLeft:SetAtlas(nameKey.."-NineSlice-CornerBottomLeft")
    SanctumFrameBottomRight:SetAtlas(nameKey.."-NineSlice-CornerBottomRight")

    -- Sanctum Reservior
    SanctumReservoirBackground:SetAtlas("CovenantSanctum-Resevoir-Empty-"..nameKey)
    SanctumReservoirFill:SetAtlas("CovenantSanctum-Resevoir-Full-"..nameKey)
    SanctumReservoirSpark:SetAtlas("CovenantSanctum-Reservoir-Spark-"..nameKey)

    -- Sanctum Upgrades
    UpgradeTravel.Border:SetAtlas("CovenantSanctum-Icon-Border-"..nameKey)
    UpgradeDiversion.Border:SetAtlas("CovenantSanctum-Icon-Border-"..nameKey)
    UpgradeAdventure.Border:SetAtlas("CovenantSanctum-Icon-Border-"..nameKey)
    UpgradeUnique.Border:SetAtlas("CovenantSanctum-Icon-Border-"..nameKey)
end

function SetupCovenantTalents()
    local characterData = AltVaultDB.characters[CharacterJournal.activeAlt]
    local talents = characterData.covenant.soulbind.talents
    local covenantData = C_Covenants.GetCovenantData(characterData.covenant.covenantID)
    local covenantName = covenantData.name
    local nameKey = covenantData.textureKit
    local shotKey = characterData.covenant.soulbind.textureKit

    if talents and shotKey then
        -- Soulbind Photo
        SoulbindShot:SetTexture("Interface\\Soulbinds\\SoulbindsShots"..SOULSHOT_NAME[nameKey])
        SoulbindShot:SetTexCoord(SOULSHOT_MAPPING[shotKey].left, SOULSHOT_MAPPING[shotKey].right, SOULSHOT_TOP, SOULSHOT_BOTTOM)
        -- Soulbind Name
        SoulbindLabel:SetText(characterData.covenant.soulbind.name)

        local powerButtons = { AltCovenantMainContent.Powers:GetChildren() }
        for row = 1, NUM_POWER_ROWS do
            local power = talents["talent"..row]
            local powerButton
            if #powerButtons == 0 then
                powerButton = CreateFrame("Button", "$parentSoulbindPower"..row, AltCovenantMainContent.Powers, "SoulbindPowerTemplate")
            else
                powerButton = powerButtons[row]
            end
            powerButton:SetSize(32, 38)
            powerButton.Ring:SetAtlas("Soulbinds_Tree_Ring", false)
            powerButton.Emblem:SetAtlas(nil);
    
            local anchorPoint = "TOPLEFT"
            local relativePoint = "BOTTOMRIGHT"
            local relativeTo = "$parentSoulbindPower".. row - 1
            if row == 1 then
                powerButton:SetPoint(anchorPoint)
            else
                if math.fmod(row,2) == 1 then
                    anchorPoint = "BOTTOMLEFT"
                    relativePoint = "TOPRIGHT"
                end
                if power and power.isConduit then
                    if power.isEnhanced then
                        powerButton.Ring:SetAtlas("Soulbinds_Tree_Conduit_Ring_Enhanced", false)
                        powerButton.RingSheen.Anim:SetPlaying(true);
                    else
                        powerButton.Ring:SetAtlas("Soulbinds_Tree_Conduit_Ring", false)
                    end
                    
                    powerButton:SetPoint(anchorPoint, relativeTo, relativePoint, -5, 0)
                    if power.conduitType then
                        local atlas = GetConduitEmblemAtlas(power.conduitType);
                        powerButton.Emblem:SetAtlas(atlas, false);
                    end
                else
                    powerButton:SetPoint(anchorPoint, relativeTo, relativePoint, -3, 0)
                end
            end
    
            if not power or power.spellID == 0 then
                powerButton.Icon:SetAtlas("Garr_MonumentLocked")
                powerButton.Ring:SetDesaturated(true)
                powerButton:SetScript("OnEnter", nil)
            else
                powerButton.Icon:SetTexture(GetSpellTexture(power.spellID));
                powerButton.Ring:SetDesaturated(false)
                powerButton:SetID(row)
                powerButton:SetScript("OnEnter", CovenantPowerButton_OnEnter)
            end
            powerButton:Show()
        end
        AltCovenantMainLocked:Hide()
        AltCovenantMainContent:Show()
    else
        AltCovenantMainContent:Hide()
        AltCovenantMainLocked:Show()
    end

end

function SetupCovenantSanctum()
    local characterData = AltVaultDB.characters[CharacterJournal.activeAlt]
    local upgrades = characterData.covenant.sanctum.upgrades

    if upgrades then
        local soulsValue = characterData.covenant.sanctum.redeemedSouls.quantity
        local animaValue = characterData.covenant.sanctum.anima.quantity
        local maxDisplayableValue = characterData.covenant.sanctum.anima.maxDisplayable
    
        local soulsLabel = GetCurrencyString(SOULS_ID, soulsValue, nil, false);
        SoulsLabel:SetText(soulsLabel)
    
        local animaLabel = GetCurrencyString(ANIMA_ID, animaValue, nil, true);
        AnimaLabel:SetText(animaLabel)
    
        local isFull = false;
        if animaValue == 0 then
            SanctumReservoirFill:Hide();
            SanctumReservoirSpark:Hide();
        else
            SanctumReservoirFill:Show();
            local totalHeight = 252;
            if animaValue >= maxDisplayableValue then
                SanctumReservoirFill:SetHeight(totalHeight);
                SanctumReservoirFill:SetTexCoord(0, 1, 0, 1);
                SanctumReservoirSpark:Hide();
                isFull = true;
            else
                local usableHeight = 136;  -- orb portion of the artwork
                local base = (totalHeight - usableHeight) / 2;
                local percent = animaValue / maxDisplayableValue;
                local height = base + usableHeight * percent;
                SanctumReservoirFill:SetHeight(height);
                local coordTop = 1 - height / totalHeight;
                SanctumReservoirFill:SetTexCoord(0, 1, coordTop, 1);
                SanctumReservoirSpark:Show();
            end
        end
    
        -- Travel
        UpgradeTravel.Icon:SetTexture(upgrades.feature1.icon)
        UpgradeTravel:SetID(1)
        if upgrades.feature1.currentTier > 0 then
            UpgradeTravel.Icon:SetDesaturated(false)
            UpgradeTravel.Tier:SetText(upgrades.feature1.currentTier)
            UpgradeTravel.Tier:Show()
            UpgradeTravel.Border:SetDesaturated(false)
            UpgradeTravel.TierBorder:Show()
        else
            UpgradeTravel.Icon:SetDesaturated(true)
            UpgradeTravel.Border:SetDesaturated(true)
            UpgradeTravel.Tier:Hide()
            UpgradeTravel.TierBorder:Hide()
        end
        -- Diversion
        UpgradeDiversion.Icon:SetTexture(upgrades.feature2.icon)
        UpgradeDiversion:SetID(2)
        if upgrades.feature2.currentTier > 0 then
            UpgradeDiversion.Icon:SetDesaturated(false)
            UpgradeDiversion.Tier:SetText(upgrades.feature2.currentTier)
            UpgradeDiversion.Tier:Show()
            UpgradeDiversion.Border:SetDesaturated(false)
            UpgradeDiversion.TierBorder:Show()
        else
            UpgradeDiversion.Icon:SetDesaturated(true)
            UpgradeDiversion.Border:SetDesaturated(true)
            UpgradeDiversion.Tier:Hide()
            UpgradeDiversion.TierBorder:Hide()
        end
        -- Adventure
        UpgradeAdventure.Icon:SetTexture(upgrades.feature3.icon)
        UpgradeAdventure:SetID(3)
        if upgrades.feature3.currentTier > 0 then
            UpgradeAdventure.Icon:SetDesaturated(false)
            UpgradeAdventure.Tier:SetText(upgrades.feature3.currentTier)
            UpgradeAdventure.Tier:Show()
            UpgradeAdventure.Border:SetDesaturated(false)
            UpgradeAdventure.TierBorder:Show()
        else
            UpgradeAdventure.Icon:SetDesaturated(true)
            UpgradeAdventure.Border:SetDesaturated(true)
            UpgradeAdventure.Tier:Hide()
            UpgradeAdventure.TierBorder:Hide()
        end
        -- Unique
        local uniqueData
        if upgrades.feature4 ~= nil then
            uniqueData = upgrades.feature4
        else
            -- No idea why this is happening
            uniqueData = upgrades.feature5
        end
        UpgradeUnique.Icon:SetTexture(uniqueData.icon)
        UpgradeUnique:SetID(4)
        if uniqueData.currentTier > 0 then
            UpgradeUnique.Icon:SetDesaturated(false)
            UpgradeUnique.Tier:SetText(uniqueData.currentTier)
            UpgradeUnique.Tier:Show()
            UpgradeUnique.Border:SetDesaturated(false)
            UpgradeUnique.TierBorder:Show()
        else
            UpgradeUnique.Icon:SetDesaturated(true)
            UpgradeUnique.Border:SetDesaturated(true)
            UpgradeUnique.Tier:Hide()
            UpgradeUnique.TierBorder:Hide()
        end
        AltCovenantSanctumLocked:Hide()
        AltVaultCurrencyShadow:Show()
        AltCovenantSanctumContent:Show()
    else
        AltCovenantSanctumContent:Hide()
        AltVaultCurrencyShadow:Hide()
        AltCovenantSanctumLocked:Show()
    end
end

function GetConduitEmblemAtlas(conduitType)
	if conduitType == Enum.SoulbindConduitType.Potency then
		return "Soulbinds_Tree_Conduit_Icon_Attack";
	elseif conduitType == Enum.SoulbindConduitType.Endurance then
		return "Soulbinds_Tree_Conduit_Icon_Protect";
	elseif conduitType == Enum.SoulbindConduitType.Finesse then
		return "Soulbinds_Tree_Conduit_Icon_Utility";
	end
end

function CovenantPowerButton_OnEnter(self)
    GameTooltip:SetOwner(self, "ANCHOR_TOP")
    local talentData = AltVaultDB.characters[CharacterJournal.activeAlt].covenant.soulbind.talents['talent'..self:GetID()]
    if talentData.isConduit then
        if talentData.isEnhanced == true then
            GameTooltip:SetEnhancedConduit(talentData.conduitID, talentData.rank);
        else
            GameTooltip:SetConduit(talentData.conduitID, talentData.rank);
        end
    elseif talentData.spellID and talentData.spellID ~= 0 then
        GameTooltip:SetSpellByID(talentData.spellID)
    end
    CursorUpdate(self);
    GameTooltip:Show();
end

function NextReward_OnEnter(self)
    GameTooltip:SetOwner(self, "ANCHOR_TOP")
    local rewardData = AltVaultDB.characters[CharacterJournal.activeAlt].covenant.nextRewards[self:GetID()]
    GameTooltip:SetText(rewardData.rewardName,1,1,1);
	GameTooltip:AddLine(rewardData.rewardDescription, nil, nil, nil, true);
    GameTooltip:Show()
    CursorUpdate(self);
end

function SanctumUpgrade_OnEnter(self)
    GameTooltip:SetOwner(self, "ANCHOR_TOPRIGHT")
    local upgradeId = self:GetID()
    local upgradeData = AltVaultDB.characters[CharacterJournal.activeAlt].covenant.sanctum.upgrades['feature'..upgradeId]
    if upgradeData == nil and upgradeId == 4 then
        upgradeData = AltVaultDB.characters[CharacterJournal.activeAlt].covenant.sanctum.upgrades['feature5']
    end
    GameTooltip:SetText(upgradeData.title,1,1,1);
	GameTooltip:AddLine(upgradeData.description, nil, nil, nil, true);
    GameTooltip:Show()
    CursorUpdate(self);
end




