local _, core = ...
local AltVault = core.AltVault

local updates = {
    {
        version = "1.6.5 - 1.7.0",
        title = "Version updates",
    },
    {
        version = "1.6.4",
        title = "Covenant Bug Fixes. 9.1 Prep",
    },
    {
        version = "1.6.0",
        title = "Covenant Addition",
        changes = {
            "Added Covenant Data Tab",
            "Added Addon Options",
            "Added ability to hide minimap",
            "Added updates section",
        }
    },
    {
        version = "1.5.5",
        title = "Storage Update",
        changes = {
            "Added Guild Vault data",
            "Consolidated Bags, Bank, and Guild Vault into new Storage tab",
            "Added container search",
        }
    },
    {
        version = "1.5.1",
        changes = {
            "Updated minimap icon to work with other addons that manage map icons."
        }
    },
    {
        version = "1.5.0",
        title = "Restructure",
    },
    {
        version = "1.0.0",
        title = "Initial Release"
    },
}

function AltVaultOptions_OnLoad(frame)
    frame.name = "AltVault"
    local version = GetAddOnMetadata("AltVault", "Version")
    frame.Title:SetText("AltVault v"..version)
    frame.ShowMinimapIcon.Text:SetText("Hide Minimap Button")
    frame.HideGoldValues.Text:SetText("Hide Gold Values")
    InterfaceOptions_AddCategory(frame);
end

function AltVaultOptions_OnShow(frame)
    frame.ShowMinimapIcon:SetChecked(AltVaultDB.minimap.profile.minimap.hide ~= false)
    frame.HideGoldValues:SetChecked(AltVaultDB.data.hideGoldValues ~= false)
    RenderUpdates()
end

function AltVaultOptionsMM_OnClick()
    if AltVaultDB.minimap.profile.minimap.hide == false then 
        AltVaultDB.minimap.profile.minimap.hide = true
        LibStub("LibDBIcon-1.0"):Hide("AltVault")
    else
        AltVaultDB.minimap.profile.minimap.hide = false
        LibStub("LibDBIcon-1.0"):Show("AltVault")
    end
end

function AltVaultOptionsGold_OnClick()
    if AltVaultDB.data.hideGoldValues == false then
        AltVaultDB.data.hideGoldValues = true
    else
        AltVaultDB.data.hideGoldValues = false
    end
end

function RenderUpdates()
    for i = 1, #updates do
        local update = updates[i];
        local titleString = AltVaultOptions:CreateFontString("UpdateTitle"..i, "ARTWORK", "GameFontHighlightMedium")
        if i == 1 then
            titleString:SetPoint("TOPLEFT", "UpdatesTitle", "BOTTOMLEFT", 10, -10)
        else
            local prevUpdate = updates[i-1]
            if prevUpdate.changes then
                titleString:SetPoint("TOPLEFT", "Update".. i-1 .."Change"..#prevUpdate.changes, "BOTTOMLEFT", -15, -15)
            else
                titleString:SetPoint("TOPLEFT", "UpdateTitle"..i-1, "BOTTOMLEFT", 0, -10)
            end
        end

        local hex = select(4, AltVault:GetThemeColor());
        local prefix = string.format("|cff%s%s|r", hex:upper(), update.version);
        local title = update.title or "Patch"
        titleString:SetText(prefix..": "..title)

        if update.changes then
            for j = 1, #update.changes do
                local change = update.changes[j]
                local changeString = AltVaultOptions:CreateFontString("Update".. i .."Change"..j, "ARTWORK", "GameFontHighlightSmall")
                if j == 1 then
                    changeString:SetPoint("TOPLEFT", "UpdateTitle"..i, "BOTTOMLEFT", 15, -8)
                else
                    changeString:SetPoint("TOPLEFT", "Update".. i .."Change"..j-1, "BOTTOMLEFT", 0, -8)
                end
                changeString:SetText("- "..change)
            end
        end
        
    end
end